/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/
#include <xc.h>         /* XC8 General Include File */
#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
//#include <stdlib.h>
//#include <stdio.h>

#include "I2Cmaster.h"

void I2C_Init (void)
{
    TRISBbits.TRISB1 = 1;      // SDA
    TRISBbits.TRISB4 = 1;      // SCL
    // I2C Master Clock Speed: 32000000 / ((4 * (SSPADD + 1)) = 16000000/(4*(79+1))
    SSPSTAT = 0xC0;      // Slew Rate is disable for 100 kHz mode
    SSPCON1 = 0x28;      // Enable SDA and SCL, I2C Master mode, clock = Freq/(4*(SSPADD+1)) = Freq_I2C
    SSPCON2 = 0x00;      // Reset MSSP Control Register
    SSPADD = 79;         // Standard I2C Clock speed: 79 -> 100 kHz & 19 -> 400kHz
}

void I2C_Idle(void)
{
    // Wait I2C Bus and Status Idle (i.e. ACKEN, RCEN, PEN, RSEN, SEN)
    while (( SSPCON2 & 0x1F ) || ( SSPSTATbits.R_nW));
}

void I2C_Start(unsigned char stype)
{
    I2C_Idle();                     // Ensure the I2C module is idle
    if (stype == I2C_START_CMD)
    {
        SSPCON2bits.SEN = 1;          // Start I2C Transmission
        while(SSPCON2bits.SEN);
    }
    else
    {
        SSPCON2bits.RSEN = 1;         // ReStart I2C Transmission
        while(SSPCON2bits.RSEN);
    }
}

void I2C_Stop(void)
{
    SSPCON2bits.PEN = 1;                // Stop I2C Transmission
    while(SSPCON2bits.PEN);
}

void I2C_Write(unsigned char data)
{
  // Send the Data to I2C Bus
  SSPBUF = data;
  if (SSPCON1bits.WCOL)         // Check for write collision
    return;

  while(SSPSTATbits.BF);        // Wait until write cycle is complete
  I2C_Idle();                   // Ensure the I2C module is idle
}

char I2C_Read(void)
{
  // Ensure the I2C module is idle
  I2C_Idle();

  // Enable Receive Mode
  SSPCON2bits.RCEN = 1;           // Enable master for 1 byte reception
  while(!SSPSTATbits.BF);         // Wait until buffer is full
  return(SSPBUF);
}