#include <xc.h>         // XC8 compilateur
#include <stdint.h>
#include <stdio.h>     // uint8_t dfinitions
#include <stdlib.h>

#include "system.h"

// Fonction d'initialisation du microcontroleur
void Init_System(void)
{
// Oscillateur configur en 32MHz)
    OSCCON = 0x70;
    WPUAbits.WPUA5 = 1;
    
// Toutes les lignes analogiques sont bloques
    ANSELA = 0x00;
    ANSELB = 0x00;
    
// Toutes les registres des ports A et B sont initialises  0;
    LATA = 0x00;
    LATB = 0x00;
    
// Toutes les lignes des ports A et B sont configures en entre
    TRISA = 0xFF;
    TRISB = 0xFF;
    
// TODO 2 -> Dfinir le sens des lignes utilises
    DIR_DEL = 0;
    DIR_BP = 1;
    DIR_MOT_DR_1 = 0;
    DIR_MOT_DR_2 = 0;
    DIR_MOT_GA_1 = 0;
    DIR_MOT_GA_2 = 0;

// TODO 3 -> Choisir le niveau par dfaut des lignes en sortie
    DEL = 0;
    MOT_DR_1 = 0;
    MOT_DR_2 = 0;
    MOT_GA_1 = 0;
    MOT_GA_2 = 0;
    
/*  Initialisation des diverses registres */    
    Init_ADC();
    SelChanADC(0);
    Init_Timer0();
    Init_Timer1();
    Init_UART();
    Init_Timer2();
    
    delayMs(200);
    printf("\r\nPicky 4.2\r\n");
    printf("Test Hardware\r\n");
}

// Fonction de temporisation en msec.
void delayMs(uint16_t ms)
{
    while((ms--))
    {
        __delay_ms(1);
    }
}

/*  Initialisation du Convertisseur Analogique Digital ADC */
void Init_ADC(void)
{
    ANSELA = 0b01000111;        // AN0, AN1, AN2 et AN6 en mode Analogique
    ADCON1bits.ADNREF = 0;      // Ref Vdd-Vss
    ADCON1bits.ADPREF = 0;
    ADCON1bits.ADFM = 0;        // Justifi a gauche (mode 8bis)
//    ADCON1bits.ADFM = 1;        // Justifi a droite (mode 10bis)
    ADCON1bits.ADCS = 0b111;    // TAD 16 TOSC (2s @32MHz)
    ADCON0bits.CHS = 0;         // Slectionne le canal 0 -> AN0
    ADCON0bits.ADON = 1;        // ADC Module enable
}

/*  Fonction qui permet de choisir la ligne pour l' ADC */
void SelChanADC(uint8_t canal)
{
    ADCON0bits.CHS = canal;
}

/*  Lecture de l'entre analogique slectionne */
uint8_t Read_Adc_8bits(void)
{
    ADCON0bits.GO_nDONE = 1;
    while (ADCON0bits.GO_nDONE);
    return ADRESH;
}

//uint16_t Read_Adc_10bits(void)
//{
//    ADCON0bits.GO = 1;
//    while (ADCON0bits.GO_nDONE);
//    return ADRESH<<8 | ADRESL;
//}

/*  Paramtrage du Timer0 */
void Init_Timer0(void)
{
    OPTION_REGbits.TMR0CS = 0;      // Compteut sur Fosc/4
    OPTION_REGbits.PSA = 0;         // Prescaler vers Timer0
    OPTION_REGbits.PS = TMR0_PRE;   // Choix du Prescaler
    TMR0 = TMR0_VALUE;              // Prchargement du Timer0
    INTCONbits.TMR0IF = 0;          // Effacement du "flag" du Timer0
}

/*  Fonction de synchronisation de la boucle infinie dans le fichier main.c */
void Synchronisation(void)
{
    while (!INTCONbits.TMR0IF);     // Attente dbordement du Timer0
    TMR0 = TMR0_VALUE;              // Prchargement du Timer0
    INTCONbits.TMR0IF = 0;          // Effacement du "flag" du Timer0
}

/*  Paramtrage du Timer1 */
void Init_Timer1(void)
{
    T1CONbits.TMR1CS = 0;           // Internal (Fosc/4))
    T1CONbits.T1CKPS = TMR1_PRE;    // Init du Prescaler
    T1CONbits.TMR1ON = 1;           // Timer1 actif
    TMR1 = TMR1_VALUE;              // 250 * 32 * 125ns = 1ms
}

/*  Initialisation de l'UART pour le dialogue C <-> PC */
void Init_UART(void)
{
    TRISBbits.TRISB5 = 0;           // ligne TX en sortie
    TRISBbits.TRISB2 = 1;           // Ligne RX en entre
    APFCON0bits.RXDTSEL = 1;        // RX to RB2
    APFCON1bits.TXCKSEL = 1;        // TX to RB5

    BAUDCONbits.BRG16 = 1;          // BRG16 = 1
    RCSTA = 0x90;                   // CREN = 1, SPEN = 1
    TXSTA = 0x24;                   // SYNC = 0; BRGH = 1
    SPBRGL = 138;                   // 57600
    SPBRGH = 0;
}

/*  Pour crire un seul caractre */
void putch(char c)
{
    while(PIR1bits.TXIF == 0);
    TXREG = c;
}

/*  Pour lire un seul caractre */
//char getch(void)
//{
//    while (!PIR1bits.RCIF);
//    return RCREG;
//}

/*  Dclarations des fonctions associes au Timer2 */
void Init_Timer2(void)
{
    T2CONbits.T2CKPS =0b10;     // Prescaler sur 16
    PR2 = 99;                  // (PR2 + 1)*prescaler*125ns -> 5kHz
    T2CONbits.TMR2ON = 1;       // Timer2 On
    
    CCP1CON = 0x1C;             // CCP1 en mode PWM
    CCPR1 = 0;
    CCP2CON = 0x1C;             // CCP2 en mode PWM
    CCPR2 = 0;
    CCP3CON = 0x1C;             // CCP3 en mode PWM
    CCPR3 = 0;
    CCP4CON = 0x1C;             // CCP4 en mode PWM
    CCPR4 = 0;
}